@@ -8,13 +8,16 @@ Current-flow closeness centrality measures.
 #    Pieter Swart <swart@lanl.gov>
 #    All rights reserved.
 #    BSD license.
-__author__ = """Aric Hagberg (hagberg@lanl.gov)"""
+__author__ = """Aric Hagberg <aric.hagberg@gmail.com>"""
 
 __all__ = ['current_flow_closeness_centrality','information_centrality']
 
 import networkx as nx
+from networkx.algorithms.centrality.flow_matrix import *
+    
 
-def current_flow_closeness_centrality(G,normalized=True):
+def current_flow_closeness_centrality(G, normalized=True, weight='weight', 
+                                      dtype=float, solver='lu'):
     """Compute current-flow closeness centrality for nodes.
 
     A variant of closeness centrality based on effective
@@ -24,12 +27,21 @@ def current_flow_closeness_centrality(G,normalized=True):
     Parameters
     ----------
     G : graph
-      A networkx graph 
+      A NetworkX graph 
 
     normalized : bool, optional
       If True the values are normalized by 1/(n-1) where n is the 
       number of nodes in G.
-       
+
+    dtype: data type (float)
+      Default data type for internal matrices.
+      Set to np.float32 for lower memory consumption.
+
+    solver: string (default='lu')
+       Type of linear solver to use for computing the flow matrix.
+       Options are "full" (uses most memory), "lu" (recommended), and 
+       "cg" (uses least memory).
+
     Returns
     -------
     nodes : dictionary
@@ -58,52 +70,51 @@ def current_flow_closeness_centrality(G,normalized=True):
        Social Networks. Volume 11, Issue 1, March 1989, pp. 1-37
        http://dx.doi.org/10.1016/0378-8733(89)90016-6
     """
+    from networkx.utils import reverse_cuthill_mckee_ordering 
     try:
         import numpy as np
     except ImportError:
-        raise ImportError("flow_closeness_centrality() requires NumPy: http://scipy.org/ ")
-    
-
+        raise ImportError('current_flow_closeness_centrality requires NumPy ',
+                          'http://scipy.org/')
+    try:
+        import scipy 
+    except ImportError:
+        raise ImportError('current_flow_closeness_centrality requires SciPy ',
+                          'http://scipy.org/')
+    if G.is_directed():
+        raise nx.NetworkXError('current_flow_closeness_centrality ',
+                               'not defined for digraphs.')
     if G.is_directed():
         raise nx.NetworkXError(\
             "current_flow_closeness_centrality() not defined for digraphs.")
-
     if not nx.is_connected(G):
         raise nx.NetworkXError("Graph not connected.")
+    solvername={"full" :FullInverseLaplacian,
+                "lu": SuperLUInverseLaplacian,
+                "cg": CGInverseLaplacian}
+    n = G.number_of_nodes()
+    ordering = list(reverse_cuthill_mckee_ordering(G))
+    # make a copy with integer labels according to rcm ordering
+    # this could be done without a copy if we really wanted to
+    H = nx.relabel_nodes(G,dict(zip(ordering,range(n))))
+    betweenness = dict.fromkeys(H,0.0) # b[v]=0 for v in H
+    n = G.number_of_nodes()
+    L = laplacian_sparse_matrix(H, nodelist=range(n), weight=weight, 
+                                dtype=dtype, format='csc')
+    C2 = solvername[solver](L, width=1, dtype=dtype) # initialize solver
+    for v in H:
+        col=C2.get_row(v)
+        for w in H:
+            betweenness[v]+=col[v]-2*col[w]
+            betweenness[w]+=col[v]
 
-    betweenness=dict.fromkeys(G,0.0) # b[v]=0 for v in G
-    n=len(G)
-    mapping=dict(zip(G,list(range(n))))  # map nodes to integers
-    C=_compute_C(G)
-    for v in G:
-        vi=mapping[v]
-        for w in G:
-            wi=mapping[w]
-            betweenness[v]+=C[vi,vi]-2*C[wi,vi]
-            betweenness[w]+=C[vi,vi]
-                
     if normalized:
         nb=len(betweenness)-1.0
     else:
         nb=1.0
-    for v in G:
+    for v in H:
         betweenness[v]=nb/(betweenness[v])
-    return betweenness            
-
-
-def _compute_C(G):
-    """Compute inverse of Laplacian."""
-    try:
-        import numpy as np
-    except ImportError:
-        raise ImportError("flow_closeness_centrality() requires NumPy: http://scipy.org/ ")
-    L=nx.laplacian(G) # use ordering of G.nodes()
-    # remove first row and column
-    LR=L[1:,1:]
-    LRinv=np.linalg.inv(LR)
-    C=np.zeros(L.shape)
-    C[1:,1:]=LRinv
-    return C
+    return dict((ordering[k],v) for k,v in betweenness.items())
 
 information_centrality=current_flow_closeness_centrality
 
