@@ -50,26 +50,54 @@ def find_cliques(G):
     find_cliques_recursive : 
     A recursive version of the same algorithm
 
+    Notes
+    -----
+    Based on the algorithm published by Bron & Kerbosch (1973) [1]_
+    as adapated by Tomita, Tanaka and Takahashi (2006) [2]_
+    and discussed in Cazals and Karande (2008) [3]_.
+
+    This algorithm ignores self-loops and parallel edges as
+    clique is not conventionally defined with such edges.
+
+    There are often many cliques in graphs.  This algorithm can
+    run out of memory for large graphs.
+
     References
     ----------
-    Based on the algorithm published by Bron & Kerbosch (1973)
-        http://doi.acm.org/10.1145/362342.362367
-    as adapated by Tomita, Tanaka and Takahashi (2006)
-        http://dx.doi.org/10.1016/j.tcs.2006.06.015
-    and discussed in Cazals and Karande (2008)
-        http://dx.doi.org/10.1016/j.tcs.2008.05.010
-     """
+    .. [1] Bron, C. and Kerbosch, J. 1973. 
+       Algorithm 457: finding all cliques of an undirected graph. 
+       Commun. ACM 16, 9 (Sep. 1973), 575-577. 
+       http://portal.acm.org/citation.cfm?doid=362342.362367
+   
+    .. [2] Etsuji Tomita, Akira Tanaka, Haruhisa Takahashi, 
+       The worst-case time complexity for generating all maximal 
+       cliques and computational experiments, 
+       Theoretical Computer Science, Volume 363, Issue 1, 
+       Computing and Combinatorics, 
+       10th Annual International Conference on 
+       Computing and Combinatorics (COCOON 2004), 25 October 2006, Pages 28-42
+       http://dx.doi.org/10.1016/j.tcs.2006.06.015
+
+    .. [3] F. Cazals, C. Karande, 
+       A note on the problem of reporting maximal cliques, 
+       Theoretical Computer Science,
+       Volume 407, Issues 1-3, 6 November 2008, Pages 564-568, 
+       http://dx.doi.org/10.1016/j.tcs.2008.05.010
+    """
+
     # Cache nbrs and find first pivot (highest degree)
     maxconn=-1
     nnbrs={}
     pivotnbrs=set() # handle empty graph
     for n,nbrs in G.adjacency_iter():
+        nbrs=set(nbrs)
+        nbrs.discard(n)
         conn = len(nbrs)
         if conn > maxconn:
-            nnbrs[n] = pivotnbrs = set(nbrs)
+            nnbrs[n] = pivotnbrs = nbrs
             maxconn = conn
         else:
-            nnbrs[n] = set(nbrs)
+            nnbrs[n] = nbrs
     # Initial setup
     cand=set(nnbrs)
     smallcand = cand - pivotnbrs
@@ -158,19 +186,42 @@ def find_cliques_recursive(G):
     --------
     find_cliques : An nonrecursive version of the same algorithm
 
-    Reference::
- 
-    Based on the algorithm published by Bron & Kerbosch (1973)
-        http://doi.acm.org/10.1145/362342.362367
-    as adapated by Tomita, Tanaka and Takahashi (2006)
-        http://dx.doi.org/10.1016/j.tcs.2006.06.015
-    and discussed in Cazals and Karande (2008)
-        http://dx.doi.org/10.1016/j.tcs.2008.05.010
- 
+    Notes
+    -----
+    Based on the algorithm published by Bron & Kerbosch (1973) [1]_
+    as adapated by Tomita, Tanaka and Takahashi (2006) [2]_
+    and discussed in Cazals and Karande (2008) [3]_.
+
+    This algorithm ignores self-loops and parallel edges as
+    clique is not conventionally defined with such edges.
+
+    References
+    ----------
+    .. [1] Bron, C. and Kerbosch, J. 1973. 
+       Algorithm 457: finding all cliques of an undirected graph. 
+       Commun. ACM 16, 9 (Sep. 1973), 575-577. 
+       http://portal.acm.org/citation.cfm?doid=362342.362367
+   
+    .. [2] Etsuji Tomita, Akira Tanaka, Haruhisa Takahashi, 
+       The worst-case time complexity for generating all maximal 
+       cliques and computational experiments, 
+       Theoretical Computer Science, Volume 363, Issue 1, 
+       Computing and Combinatorics, 
+       10th Annual International Conference on 
+       Computing and Combinatorics (COCOON 2004), 25 October 2006, Pages 28-42
+       http://dx.doi.org/10.1016/j.tcs.2006.06.015
+
+    .. [3] F. Cazals, C. Karande, 
+       A note on the problem of reporting maximal cliques, 
+       Theoretical Computer Science,
+       Volume 407, Issues 1-3, 6 November 2008, Pages 564-568, 
+       http://dx.doi.org/10.1016/j.tcs.2008.05.010
     """
     nnbrs={}
     for n,nbrs in G.adjacency_iter():
-        nnbrs[n]=set(nbrs)
+        nbrs=set(nbrs)
+        nbrs.discard(n)
+        nnbrs[n]=nbrs
     if not nnbrs: return [] # empty graph
     cand=set(nnbrs)
     done=set()
@@ -230,8 +281,7 @@ def make_max_clique_graph(G,create_using=None,name=None):
     This should be the same as make_clique_bipartite followed
     by project_up, but it saves all the intermediate steps.
     """
-    cliq=map(set,find_cliques(G))
-    size=len(cliq)
+    cliq=list(map(set,find_cliques(G)))
     if create_using:
         B=create_using
         B.clear()
@@ -379,7 +429,19 @@ def node_clique_number(G,nodes=None,cliques=None):
         Optional list of cliques can be input if already computed.
 
     """
-    if cliques is None:
+    if cliques is None: 
+        if nodes is not None:
+            # Use ego_graph to decrease size of graph
+            if isinstance(nodes,list):
+                d={}
+                for n in nodes:
+                    H=networkx.ego_graph(G,n)
+                    d[n]=max( (len(c) for c in find_cliques(H)) )
+            else:
+                H=networkx.ego_graph(G,nodes)
+                d=max( (len(c) for c in find_cliques(H)) )
+            return d
+        # nodes is None--find all cliques
         cliques=list(find_cliques(G))
 
     if nodes is None:
