@@ -1,6 +1,7 @@
 #!/usr/bin/env python
 from nose.tools import *
 from nose import SkipTest
+from nose.plugins.attrib import attr
 import networkx
 
 # Example from
@@ -36,7 +37,19 @@ class TestHITS:
         for n in G:
             assert_almost_equal(a[n],G.a[n],places=4)
 
+    def test_hits_nstart(self):
+        G = self.G
+        nstart = dict([(i, 1./2) for i in G])
+        h, a = networkx.hits(G, nstart = nstart)
+
+    @attr('numpy')
     def test_hits_numpy(self):
+        try:
+            import numpy as np
+        except ImportError:
+            raise SkipTest('NumPy not available.')
+
+
         G=self.G
         h,a=networkx.hits_numpy(G)
         for n in G:
@@ -46,6 +59,11 @@ class TestHITS:
 
 
     def test_hits_scipy(self):
+        try:
+            import scipy as sp
+        except ImportError:
+            raise SkipTest('SciPy not available.')
+
         G=self.G
         h,a=networkx.hits_scipy(G,tol=1.e-08)
         for n in G:
