@@ -55,6 +55,7 @@ class TestAStar:
 
     def test_astar_undirected(self):
         GG=self.XG.to_undirected()
+        GG['y']['v']['weight'] = 2
         assert nx.astar_path(GG,'s','v')==['s', 'x', 'u', 'v']
         assert nx.astar_path_length(GG,'s','v')==8
 
@@ -102,7 +103,9 @@ class TestAStar:
 
     def test_astar_w1(self):
         G=nx.DiGraph() 
-        G.add_edges_from([('s','u'), ('s','x'), ('u','v'), ('u','x'), ('v','y'), ('x','u'), ('x','v'), ('x','y'), ('y','s'), ('y','v')])
+        G.add_edges_from([('s','u'), ('s','x'), ('u','v'), ('u','x'),
+            ('v','y'), ('x','u'), ('x','w'), ('w', 'v'), ('x','y'),
+            ('y','s'), ('y','v')])
         assert nx.astar_path(G,'s','v')==['s', 'u', 'v']
         assert nx.astar_path_length(G,'s','v')== 2
 
