@@ -9,6 +9,8 @@ def test_smetric():
     g.add_edge(2,3)
     g.add_edge(2,4)
     g.add_edge(1,4)
-    sm = nx.s_metric(g)
-    assert_equal(sm, 19)
+    sm = nx.s_metric(g,normalized=False)
+    assert_equal(sm, 19.0)
+#    smNorm = nx.s_metric(g,normalized=True)
+#    assert_equal(smNorm, 0.95)
 
