@@ -455,9 +455,9 @@ class DiGraph(Graph):
         -----
         Adding an edge that already exists updates the edge data.
 
-        NetworkX algorithms designed for weighted graphs use as
-        the edge weight a numerical value assigned to the keyword
-        'weight'.
+        Many NetworkX algorithms designed for weighted graphs use as
+        the edge weight a numerical value assigned to a keyword
+        which by default is 'weight'.
 
         Examples
         --------
@@ -662,14 +662,14 @@ class DiGraph(Graph):
         neighbors_iter() and successors_iter() are the same.
         """
         try:
-            return iter(self.succ[n].keys())
+            return iter(self.succ[n])
         except KeyError:
             raise NetworkXError("The node %s is not in the digraph."%(n,))
 
     def predecessors_iter(self,n):
         """Return an iterator over predecessor nodes of n."""
         try:
-            return iter(self.pred[n].keys())
+            return iter(self.pred[n])
         except KeyError:
             raise NetworkXError("The node %s is not in the digraph."%(n,))
 
@@ -731,7 +731,7 @@ class DiGraph(Graph):
 
         """
         if nbunch is None:
-            nodes_nbrs=iter(self.adj.items())
+            nodes_nbrs=self.adj.items()
         else:
             nodes_nbrs=((n,self.adj[n]) for n in self.nbunch_iter(nbunch))
         if data:
@@ -768,7 +768,7 @@ class DiGraph(Graph):
         edges_iter : return an iterator of edges
         """
         if nbunch is None:
-            nodes_nbrs=iter(self.pred.items())
+            nodes_nbrs=self.pred.items()
         else:
             nodes_nbrs=((n,self.pred[n]) for n in self.nbunch_iter(nbunch))
         if data:
@@ -789,7 +789,7 @@ class DiGraph(Graph):
         """
         return list(self.in_edges_iter(nbunch, data))
 
-    def degree_iter(self, nbunch=None, weighted=False):
+    def degree_iter(self, nbunch=None, weight=None):
         """Return an iterator for (node, degree).
 
         The node degree is the number of edges adjacent to the node.
@@ -799,8 +799,11 @@ class DiGraph(Graph):
         nbunch : iterable container, optional (default=all nodes)
             A container of nodes.  The container will be iterated
             through once.
-        weighted : bool, optional (default=False)
-           If True return the sum of edge weights adjacent to the node.
+
+        weight : string or None, optional (default=None)
+           The edge attribute that holds the numerical value used 
+           as a weight.  If None, then each edge has weight 1.
+           The degree is the sum of the edge weights adjacent to the node.
 
         Returns
         -------
@@ -828,18 +831,18 @@ class DiGraph(Graph):
                 ((n,self.succ[n]) for n in self.nbunch_iter(nbunch)),
                 ((n,self.pred[n]) for n in self.nbunch_iter(nbunch)))
 
-        if weighted:                        
+        if weight is None:
+            for (n,succ),(n2,pred) in nodes_nbrs:
+                yield (n,len(succ)+len(pred))
+        else:
         # edge weighted graph - degree is sum of edge weights
             for (n,succ),(n2,pred) in nodes_nbrs:
-               yield (n, 
-                      sum((succ[nbr].get('weight',1) for nbr in succ))+
-                      sum((pred[nbr].get('weight',1) for nbr in pred)))
-        else:
-            for (n,succ),(n2,pred) in nodes_nbrs:
-                yield (n,len(succ)+len(pred)) 
+               yield (n,
+                      sum((succ[nbr].get(weight,1) for nbr in succ))+
+                      sum((pred[nbr].get(weight,1) for nbr in pred)))
 
 
-    def in_degree_iter(self, nbunch=None, weighted=False):
+    def in_degree_iter(self, nbunch=None, weight=None):
         """Return an iterator for (node, in-degree).
 
         The node in-degree is the number of edges pointing in to the node.
@@ -849,8 +852,11 @@ class DiGraph(Graph):
         nbunch : iterable container, optional (default=all nodes)
             A container of nodes.  The container will be iterated
             through once.
-        weighted : bool, optional (default=False)
-           If True return the sum of edge weights adjacent to the node.
+
+        weight : string or None, optional (default=None)
+           The edge attribute that holds the numerical value used 
+           as a weight.  If None, then each edge has weight 1.
+           The degree is the sum of the edge weights adjacent to the node.
 
         Returns
         -------
@@ -872,20 +878,20 @@ class DiGraph(Graph):
 
         """
         if nbunch is None:
-            nodes_nbrs=iter(self.pred.items())
+            nodes_nbrs=self.pred.items()
         else:
             nodes_nbrs=((n,self.pred[n]) for n in self.nbunch_iter(nbunch))
 
-        if weighted:
-        # edge weighted graph - degree is sum of edge weights
-            for n,nbrs in nodes_nbrs:
-                yield (n, sum(data.get('weight',1) for data in nbrs.values()))
-        else:
+        if weight is None:
             for n,nbrs in nodes_nbrs:
                 yield (n,len(nbrs))
+        else:
+        # edge weighted graph - degree is sum of edge weights
+            for n,nbrs in nodes_nbrs:
+                yield (n, sum(data.get(weight,1) for data in nbrs.values()))
 
 
-    def out_degree_iter(self, nbunch=None, weighted=False):
+    def out_degree_iter(self, nbunch=None, weight=None):
         """Return an iterator for (node, out-degree).
 
         The node out-degree is the number of edges pointing out of the node.
@@ -895,8 +901,11 @@ class DiGraph(Graph):
         nbunch : iterable container, optional (default=all nodes)
             A container of nodes.  The container will be iterated
             through once.
-        weighted : bool, optional (default=False)
-           If True return the sum of edge weights adjacent to the node.
+
+        weight : string or None, optional (default=None)
+           The edge attribute that holds the numerical value used 
+           as a weight.  If None, then each edge has weight 1.
+           The degree is the sum of the edge weights adjacent to the node.
 
         Returns
         -------
@@ -918,20 +927,20 @@ class DiGraph(Graph):
 
         """
         if nbunch is None:
-            nodes_nbrs=iter(self.succ.items())
+            nodes_nbrs=self.succ.items()
         else:
             nodes_nbrs=((n,self.succ[n]) for n in self.nbunch_iter(nbunch))
 
-        if weighted:
-        # edge weighted graph - degree is sum of edge weights
-            for n,nbrs in nodes_nbrs:
-                yield (n, sum(data.get('weight',1) for data in nbrs.values()))
-        else:
+        if weight is None:
             for n,nbrs in nodes_nbrs:
                 yield (n,len(nbrs))
+        else:
+        # edge weighted graph - degree is sum of edge weights
+            for n,nbrs in nodes_nbrs:
+                yield (n, sum(data.get(weight,1) for data in nbrs.values()))
 
 
-    def in_degree(self, nbunch=None, weighted=False):
+    def in_degree(self, nbunch=None, weight=None):
         """Return the in-degree of a node or nodes.
 
         The node in-degree is the number of edges pointing in to the node.
@@ -941,8 +950,11 @@ class DiGraph(Graph):
         nbunch : iterable container, optional (default=all nodes)
             A container of nodes.  The container will be iterated
             through once.
-        weighted : bool, optional (default=False)
-           If True return the sum of edge weights adjacent to the node.
+
+        weight : string or None, optional (default=None)
+           The edge attribute that holds the numerical value used 
+           as a weight.  If None, then each edge has weight 1.
+           The degree is the sum of the edge weights adjacent to the node.
 
         Returns
         -------
@@ -966,11 +978,11 @@ class DiGraph(Graph):
         [0, 1]
         """
         if nbunch in self:      # return a single node
-            return next(self.in_degree_iter(nbunch,weighted=weighted))[1]
+            return next(self.in_degree_iter(nbunch,weight))[1]
         else:           # return a dict
-            return dict(self.in_degree_iter(nbunch,weighted=weighted))
+            return dict(self.in_degree_iter(nbunch,weight))
 
-    def out_degree(self, nbunch=None, weighted=False):
+    def out_degree(self, nbunch=None, weight=None):
         """Return the out-degree of a node or nodes.
 
         The node out-degree is the number of edges pointing out of the node.
@@ -980,8 +992,11 @@ class DiGraph(Graph):
         nbunch : iterable container, optional (default=all nodes)
             A container of nodes.  The container will be iterated
             through once.
-        weighted : bool, optional (default=False)
-           If True return the sum of edge weights adjacent to the node.
+
+        weight : string or None, optional (default=None)
+           The edge attribute that holds the numerical value used 
+           as a weight.  If None, then each edge has weight 1.
+           The degree is the sum of the edge weights adjacent to the node.
 
         Returns
         -------
@@ -1003,9 +1018,9 @@ class DiGraph(Graph):
 
         """
         if nbunch in self:      # return a single node
-            return next(self.out_degree_iter(nbunch,weighted=weighted))[1]
+            return next(self.out_degree_iter(nbunch,weight))[1]
         else:           # return a dict
-            return dict(self.out_degree_iter(nbunch,weighted=weighted))
+            return dict(self.out_degree_iter(nbunch,weight))
 
     def clear(self):
         """Remove all nodes and edges from the graph.
@@ -1145,11 +1160,11 @@ class DiGraph(Graph):
         """
         if copy:
             H = self.__class__(name="Reverse of (%s)"%self.name)
-            H.pred=self.succ.copy()
-            H.adj=self.pred.copy()
-            H.succ=H.adj
-            H.graph=self.graph.copy()
-            H.node=self.node.copy()
+            H.add_nodes_from(self)
+            H.add_edges_from( (v,u,deepcopy(d)) for u,v,d 
+                              in self.edges(data=True) )
+            H.graph=deepcopy(self.graph)
+            H.node=deepcopy(self.node)
         else:
             self.pred,self.succ=self.succ,self.pred
             self.adj=self.succ
