@@ -370,7 +370,7 @@ class MultiDiGraph(MultiGraph,DiGraph):
 
         """
         if nbunch is None:
-            nodes_nbrs = iter(self.adj.items())
+            nodes_nbrs = self.adj.items()
         else:
             nodes_nbrs=((n,self.adj[n]) for n in self.nbunch_iter(nbunch))
         if data:
@@ -449,7 +449,7 @@ class MultiDiGraph(MultiGraph,DiGraph):
         edges_iter : return an iterator of edges
         """
         if nbunch is None:
-            nodes_nbrs=iter(self.pred.items())
+            nodes_nbrs=self.pred.items()
         else:
             nodes_nbrs=((n,self.pred[n]) for n in self.nbunch_iter(nbunch))
         if data:
@@ -494,7 +494,7 @@ class MultiDiGraph(MultiGraph,DiGraph):
         return list(self.in_edges_iter(nbunch, keys=keys, data=data))
 
 
-    def degree_iter(self, nbunch=None, weighted=False):
+    def degree_iter(self, nbunch=None, weight=None):
         """Return an iterator for (node, degree).
 
         The node degree is the number of edges adjacent to the node.
@@ -504,8 +504,11 @@ class MultiDiGraph(MultiGraph,DiGraph):
         nbunch : iterable container, optional (default=all nodes)
             A container of nodes.  The container will be iterated
             through once.
-        weighted : bool, optional (default=False)
-           If True return the sum of edge weights adjacent to the node.
+
+        weight : string or None, optional (default=None)
+           The edge attribute that holds the numerical value used 
+           as a weight.  If None, then each edge has weight 1.
+           The degree is the sum of the edge weights.
 
         Returns
         -------
@@ -532,24 +535,25 @@ class MultiDiGraph(MultiGraph,DiGraph):
             nodes_nbrs=zip(
                 ((n,self.succ[n]) for n in self.nbunch_iter(nbunch)),
                 ((n,self.pred[n]) for n in self.nbunch_iter(nbunch)))
-        if weighted:
-        # edge weighted graph - degree is sum of nbr edge weights
-            for (n,succ),(n2,pred) in nodes_nbrs:
-                deg = sum([d.get('weight',1)
-                           for data in pred.values()
-                           for d in data.values()])
-                deg += sum([d.get('weight',1) 
-                           for data in succ.values()
-                           for d in data.values()])
-                yield (n, deg)
-        else:
+
+        if weight is None:
             for (n,succ),(n2,pred) in nodes_nbrs:
                 indeg = sum([len(data) for data in pred.values()])
                 outdeg = sum([len(data) for data in succ.values()])
-                yield (n, indeg + outdeg)  
+                yield (n, indeg + outdeg)
+        else:
+        # edge weighted graph - degree is sum of nbr edge weights
+            for (n,succ),(n2,pred) in nodes_nbrs:
+                deg = sum([d.get(weight,1)
+                           for data in pred.values()
+                           for d in data.values()])
+                deg += sum([d.get(weight,1)
+                           for data in succ.values()
+                           for d in data.values()])
+                yield (n, deg)
 
 
-    def in_degree_iter(self, nbunch=None, weighted=False):
+    def in_degree_iter(self, nbunch=None, weight=None):
         """Return an iterator for (node, in-degree).
 
         The node in-degree is the number of edges pointing in to the node.
@@ -559,8 +563,11 @@ class MultiDiGraph(MultiGraph,DiGraph):
         nbunch : iterable container, optional (default=all nodes)
             A container of nodes.  The container will be iterated
             through once.
-        weighted : bool, optional (default=False)
-           If True return the sum of edge weights adjacent to the node.
+
+        weight : string or None, optional (default=None)
+           The edge attribute that holds the numerical value used 
+           as a weight.  If None, then each edge has weight 1.
+           The degree is the sum of the edge weights adjacent to the node.
 
         Returns
         -------
@@ -582,23 +589,23 @@ class MultiDiGraph(MultiGraph,DiGraph):
 
         """
         if nbunch is None:
-            nodes_nbrs=iter(self.pred.items())
+            nodes_nbrs=self.pred.items()
         else:
             nodes_nbrs=((n,self.pred[n]) for n in self.nbunch_iter(nbunch))
 
-        if weighted:
+        if weight is None:
+            for n,nbrs in nodes_nbrs:
+                yield (n, sum([len(data) for data in nbrs.values()]) )
+        else:
             # edge weighted graph - degree is sum of nbr edge weights
             for n,pred in nodes_nbrs:
-                deg = sum([d.get('weight',1)
+                deg = sum([d.get(weight,1)
                            for data in pred.values()
                            for d in data.values()])
                 yield (n, deg)
-        else:
-            for n,nbrs in nodes_nbrs:
-                yield (n, sum([len(data) for data in nbrs.values()]) )
 
 
-    def out_degree_iter(self, nbunch=None, weighted=False):
+    def out_degree_iter(self, nbunch=None, weight=None):
         """Return an iterator for (node, out-degree).
 
         The node out-degree is the number of edges pointing out of the node.
@@ -608,8 +615,11 @@ class MultiDiGraph(MultiGraph,DiGraph):
         nbunch : iterable container, optional (default=all nodes)
             A container of nodes.  The container will be iterated
             through once.
-        weighted : bool, optional (default=False)
-           If True return the sum of edge weights adjacent to the node.
+
+        weight : string or None, optional (default=None)
+           The edge attribute that holds the numerical value used 
+           as a weight.  If None, then each edge has weight 1.
+           The degree is the sum of the edge weights.
 
         Returns
         -------
@@ -631,19 +641,19 @@ class MultiDiGraph(MultiGraph,DiGraph):
 
         """
         if nbunch is None:
-            nodes_nbrs=iter(self.succ.items())
+            nodes_nbrs=self.succ.items()
         else:
             nodes_nbrs=((n,self.succ[n]) for n in self.nbunch_iter(nbunch))
 
-        if weighted:
+        if weight is None:
+            for n,nbrs in nodes_nbrs:
+                yield (n, sum([len(data) for data in nbrs.values()]) )
+        else:
             for n,succ in nodes_nbrs:
-                deg = sum([d.get('weight',1)
+                deg = sum([d.get(weight,1)
                            for data in succ.values()
                            for d in data.values()])
                 yield (n, deg)
-        else:
-            for n,nbrs in nodes_nbrs:
-                yield (n, sum([len(data) for data in nbrs.values()]) )
 
     def is_multigraph(self):
         """Return True if graph is a multigraph, False otherwise."""
@@ -812,3 +822,29 @@ class MultiDiGraph(MultiGraph,DiGraph):
             H.node[n]=self.node[n]
         H.graph=self.graph
         return H
+
+    def reverse(self, copy=True):
+        """Return the reverse of the graph.
+
+        The reverse is a graph with the same nodes and edges
+        but with the directions of the edges reversed.
+
+        Parameters
+        ----------
+        copy : bool optional (default=True)
+            If True, return a new DiGraph holding the reversed edges.
+            If False, reverse the reverse graph is created using
+            the original graph (this changes the original graph).
+        """
+        if copy:
+            H = self.__class__(name="Reverse of (%s)"%self.name)
+            H.add_nodes_from(self)
+            H.add_edges_from( (v,u,k,deepcopy(d)) for u,v,k,d 
+                              in self.edges(keys=True, data=True) )
+            H.graph=deepcopy(self.graph)
+            H.node=deepcopy(self.node)
+        else:
+            self.pred,self.succ=self.succ,self.pred
+            self.adj=self.succ
+            H=self
+        return H
