@@ -18,15 +18,15 @@ Pygraphviz: http://networkx.lanl.gov/pygraphviz
 
 """
 __author__ = """Aric Hagberg (hagberg@lanl.gov)"""
-#    Copyright (C) 2004-2008 by 
+#    Copyright (C) 2004-2008 by
 #    Aric Hagberg <hagberg@lanl.gov>
 #    Dan Schult <dschult@colgate.edu>
 #    Pieter Swart <swart@lanl.gov>
 #    All rights reserved.
 #    BSD license.
 
-__all__ = ['from_agraph', 'to_agraph', 
-           'write_dot', 'read_dot', 
+__all__ = ['from_agraph', 'to_agraph',
+           'write_dot', 'read_dot',
            'graphviz_layout',
            'pygraphviz_layout',
            'view_pygraphviz']
@@ -34,9 +34,9 @@ __all__ = ['from_agraph', 'to_agraph',
 import os
 import sys
 import tempfile
-import subprocess
+
 import networkx as nx
-from networkx.utils import _get_fh,is_string_like
+
 
 def from_agraph(A,create_using=None):
     """Return a NetworkX Graph or DiGraph from a PyGraphviz graph.
@@ -45,8 +45,8 @@ def from_agraph(A,create_using=None):
     ----------
     A : PyGraphviz AGraph
       A graph created with PyGraphviz
-      
-    create_using : NetworkX graph class instance      
+
+    create_using : NetworkX graph class instance
       The output is created using the given graph class instance
 
     Examples
@@ -70,7 +70,7 @@ def from_agraph(A,create_using=None):
     attribute or the value 1 if no edge weight attribute is found.
 
     """
-    if create_using is None:        
+    if create_using is None:
         if A.is_directed():
             if A.is_strict():
                 create_using=nx.DiGraph()
@@ -82,34 +82,32 @@ def from_agraph(A,create_using=None):
             else:
                 create_using=nx.MultiGraph()
 
-    # assign defaults        
+    # assign defaults
     N=nx.empty_graph(0,create_using)
     N.name=str(A)
-    node_attr={}
     # add nodes, attributes to N.node_attr
     for n in A.nodes():
-        N.add_node(str(n),**dict(n.attr))
+        str_attr=dict((str(k),v) for k,v in n.attr.items())
+        N.add_node(str(n),**str_attr)
 
     # add edges, assign edge data as dictionary of attributes
     for e in A.edges():
         u,v=str(e[0]),str(e[1])
         attr=dict(e.attr)
-        if N.is_multigraph():
-            if e.key is not None:
-                attr[key]=e.key
-            N.add_edge(u,v,**attr)
+        str_attr=dict((str(k),v) for k,v in attr.items())
+        if not N.is_multigraph():
+            if e.name is not None:
+                str_attr['key']=e.name
+            N.add_edge(u,v,**str_attr)
         else:
-            if e.key is not None:
-                N.add_edge(u,v,e.key,**attr)
-            else:
-                N.add_edge(u,v,**attr)
-        
-    # add default attributes for graph, nodes, and edges       
+            N.add_edge(u,v,key=e.name,**str_attr)
+
+    # add default attributes for graph, nodes, and edges
     # hang them on N.graph_attr
     N.graph['graph']=dict(A.graph_attr)
     N.graph['node']=dict(A.node_attr)
     N.graph['edge']=dict(A.edge_attr)
-    return N        
+    return N
 
 def to_agraph(N):
     """Return a pygraphviz graph from a NetworkX graph N.
@@ -118,7 +116,7 @@ def to_agraph(N):
     ----------
     N : NetworkX graph
       A graph created with NetworkX
-      
+
     Examples
     --------
     >>> K5=nx.complete_graph(5)
@@ -134,13 +132,12 @@ def to_agraph(N):
     try:
         import pygraphviz
     except ImportError:
-        raise ImportError, \
-          "to_agraph() requires pygraphviz: http://nx.lanl.gov/pygraphviz"
+        raise ImportError("to_agraph() requires pygraphviz: http://nx.lanl.gov/pygraphviz (not available for Python3")
     directed=N.is_directed()
-    strict=N.number_of_selfloops()==0 and not N.is_multigraph() 
+    strict=N.number_of_selfloops()==0 and not N.is_multigraph()
     A=pygraphviz.AGraph(name=N.name,strict=strict,directed=directed)
 
-    # default graph attributes            
+    # default graph attributes
     A.graph_attr.update(N.graph.get('graph',{}))
     A.node_attr.update(N.graph.get('node',{}))
     A.edge_attr.update(N.graph.get('edge',{}))
@@ -153,11 +150,11 @@ def to_agraph(N):
 
     if N.is_multigraph():
         for u,v,key,edgedata in N.edges_iter(data=True,keys=True):
-            str_edgedata=dict((k,str(v)) for k,v in edgedata.iteritems())
+            str_edgedata=dict((k,str(v)) for k,v in edgedata.items())
             A.add_edge(u,v,key=str(key),**str_edgedata)
     else:
         for u,v,edgedata in N.edges_iter(data=True):
-            str_edgedata=dict((k,str(v)) for k,v in edgedata.iteritems())
+            str_edgedata=dict((k,str(v)) for k,v in edgedata.items())
             A.add_edge(u,v,**str_edgedata)
 
 
@@ -171,17 +168,17 @@ def write_dot(G,path):
     G : graph
        A networkx graph
     path : filename
-       Filename or file handle to write.  
+       Filename or file handle to write.
 
     """
     try:
         import pygraphviz
     except ImportError:
-        raise ImportError, \
-          "write_dot() requires pygraphviz: http://networkx.lanl.gov/pygraphviz"
+        raise ImportError("write_dot() requires pygraphviz: http://networkx.lanl.gov/pygraphviz (not available for Python3")
 
     A=to_agraph(G)
     A.write(path)
+    A.clear()
     return
 
 def read_dot(path):
@@ -195,8 +192,7 @@ def read_dot(path):
     try:
         import pygraphviz
     except ImportError:
-        raise ImportError, \
-          "read_dot() requires pygraphviz: http://networkx.lanl.gov/pygraphviz"
+        raise ImportError("read_dot() requires pygraphviz: http://networkx.lanl.gov/pygraphviz (not available for Python3)")
     A=pygraphviz.AGraph(file=path)
     return from_agraph(A)
 
@@ -209,13 +205,13 @@ def graphviz_layout(G,prog='neato',root=None, args=''):
     G : NetworkX graph
       A graph created with NetworkX
     prog : string
-      Name of Graphviz layout program 
+      Name of Graphviz layout program
     root : string, optional
       Root node for twopi layout
     args : string, optional
       Extra arguments to Graphviz layout program
 
-    Returns : dictionary      
+    Returns : dictionary
       Dictionary of x,y, positions keyed by node.
 
     Examples
@@ -223,7 +219,7 @@ def graphviz_layout(G,prog='neato',root=None, args=''):
     >>> G=nx.petersen_graph()
     >>> pos=nx.graphviz_layout(G)
     >>> pos=nx.graphviz_layout(G,prog='dot')
-    
+
     Notes
     -----
     This is a wrapper for pygraphviz_layout.
@@ -239,13 +235,13 @@ def pygraphviz_layout(G,prog='neato',root=None, args=''):
     G : NetworkX graph
       A graph created with NetworkX
     prog : string
-      Name of Graphviz layout program 
+      Name of Graphviz layout program
     root : string, optional
       Root node for twopi layout
     args : string, optional
       Extra arguments to Graphviz layout program
 
-    Returns : dictionary      
+    Returns : dictionary
       Dictionary of x,y, positions keyed by node.
 
     Examples
@@ -253,13 +249,12 @@ def pygraphviz_layout(G,prog='neato',root=None, args=''):
     >>> G=nx.petersen_graph()
     >>> pos=nx.graphviz_layout(G)
     >>> pos=nx.graphviz_layout(G,prog='dot')
-    
+
     """
     try:
         import pygraphviz
     except ImportError:
-        raise ImportError, \
-          "pygraphviz_layout() requires pygraphviz: http://networkx.lanl.gov/pygraphviz"
+        raise ImportError("pygraphviz_layout() requires pygraphviz: http://networkx.lanl.gov/pygraphviz (not available for Python3")
     A=to_agraph(G)
     if root is not None:
         args+="-Groot=%s"%root
@@ -271,12 +266,13 @@ def pygraphviz_layout(G,prog='neato',root=None, args=''):
             xx,yy=node.attr["pos"].split(',')
             node_pos[n]=(float(xx),float(yy))
         except:
-            print "no position for node",n
+            print("no position for node",n)
             node_pos[n]=(0.0,0.0)
     return node_pos
 
-def view_pygraphviz(G, edgelabel=None, prog='neato', args='', 
-                       suffix='', filename=None):
+@nx.utils.open_file(5, 'w')
+def view_pygraphviz(G, edgelabel=None, prog='dot', args='',
+                       suffix='', path=None):
     """Views the graph G using the specified layout algorithm.
 
     Parameters
@@ -285,7 +281,7 @@ def view_pygraphviz(G, edgelabel=None, prog='neato', args='',
         The machine to draw.
     edgelabel : str, callable, None
         If a string, then it specifes the edge attribute to be displayed
-        on the edge labels. If a callable, then it is called for each 
+        on the edge labels. If a callable, then it is called for each
         edge and it should return the string to be displayed on the edges.
         The function signature of `edgelabel` should be edgelabel(data),
         where `data` is the edge attribute dictionary.
@@ -296,23 +292,23 @@ def view_pygraphviz(G, edgelabel=None, prog='neato', args='',
     suffix : str
         If `filename` is None, we save to a temporary file.  The value of
         `suffix` will appear at the tail end of the temporary filename.
-    filename : str, None
+    path : str, None
         The filename used to save the image.  If None, save to a temporary
         file.  File formats are the same as those from pygraphviz.agraph.draw.
 
     Returns
     -------
-    filename : str
+    path : str
         The filename of the generated image.
     A : PyGraphviz graph
         The PyGraphviz graph instance used to generate the image.
 
     Notes
     -----
-    If this function is called in succession too quickly, sometimes the 
+    If this function is called in succession too quickly, sometimes the
     image is not displayed. So you might consider time.sleep(.5) between
     calls if you experience problems.
-    
+
     """
     if not len(G):
         raise nx.NetworkXException("An empty graph cannot be drawn.")
@@ -330,7 +326,7 @@ def view_pygraphviz(G, edgelabel=None, prog='neato', args='',
     for attr in attrs:
         if attr not in G.graph:
             G.graph[attr] = {}
-    
+
     # These are the default values.
     edge_attrs = {'fontsize': '10'}
     node_attrs = {'style': 'filled',
@@ -343,7 +339,7 @@ def view_pygraphviz(G, edgelabel=None, prog='neato', args='',
     def update_attrs(which, attrs):
         # Update graph attributes. Return list of those which were added.
         added = []
-        for k,v in attrs.iteritems():
+        for k,v in attrs.items():
             if k not in G.graph[which]:
                 G.graph[which][k] = v
                 added.append(k)
@@ -356,9 +352,9 @@ def view_pygraphviz(G, edgelabel=None, prog='neato', args='',
             del G.graph[which]
 
     # Update all default values
-    added_edge = update_attrs('edge', edge_attrs)
-    added_node = update_attrs('node', node_attrs)
-    added_graph = update_attrs('graph', graph_attrs)
+    update_attrs('edge', edge_attrs)
+    update_attrs('node', node_attrs)
+    update_attrs('graph', graph_attrs)
 
     # Convert to agraph, so we inherit default values
     A = to_agraph(G)
@@ -387,20 +383,20 @@ def view_pygraphviz(G, edgelabel=None, prog='neato', args='',
                 edge = A.get_edge(u,v)
                 edge.attr['label'] = str(func(data))
 
-    if filename is None:
+    if path is None:
         ext = 'png'
         if suffix:
             suffix = '_%s.%s' % (suffix, ext)
         else:
             suffix = '.%s' % (ext,)
-        fd, filename = tempfile.mkstemp(suffix=suffix)
-        path = (fd, filename)
+        path = tempfile.NamedTemporaryFile(suffix=suffix, delete=False)
     else:
-        path = (filename,)
+        # Assume the decorator worked and it is a file-object.
+        pass
 
     display_pygraphviz(A, path=path, prog=prog, args=args)
 
-    return filename, A
+    return path.name, A
 
 def display_pygraphviz(graph, path, format=None, prog=None, args=''):
     """Internal function to display a graph in OS dependent manner.
@@ -409,10 +405,8 @@ def display_pygraphviz(graph, path, format=None, prog=None, args=''):
     ----------
     graph : PyGraphviz graph
         A PyGraphviz AGraph instance.
-    path : tuple
-        For temporary files, path is a 2-tuple containing the file descriptor
-        as returned by os.open and the filename. If `path` is a 1-tuple, then
-        the only element is the filename.
+    path :  file object
+        An already opened file object that will be closed.
     format : str, None
         An attempt is made to guess the output format based on the extension
         of the filename. If that fails, the value of `format` is used.
@@ -423,29 +417,28 @@ def display_pygraphviz(graph, path, format=None, prog=None, args=''):
 
     Notes
     -----
-    If this function is called in succession too quickly, sometimes the 
+    If this function is called in succession too quickly, sometimes the
     image is not displayed. So you might consider time.sleep(.5) between
     calls if you experience problems.
-    
+
     """
-    # This would be more useful as part of PyGraphviz.
-
-    if len(path) == 2:
-        fd, filename = path
-        path = os.fdopen(fd, "w+b")  # grab file-object associated to fd
-    else:
-        filename = path[0]
-        path = filename              # graph.draw() will open the file-object
-
-    # If we are using a temporary file, the file-object does not contain the
-    # name of the file. This means that graph.draw() will fail to detect the 
-    # format and will set it to 'dot' by default. Since we have the filename,
-    # we explicitly set the format.
     if format is None:
-        format = os.path.splitext(filename)[-1].lower()[1:]
-    
-    # Save to a file and display in the default viewer.
-    cmds = {'darwin': 'open', 'linux2': 'xdg-open', 'win32': 'start'}
-    graph.draw(path, format, prog, args)
-    subprocess.call([cmds[sys.platform], filename])
+        filename = path.name
+        format = os.path.splitext(filename)[1].lower()[1:]
+    if not format:
+        # Let the draw() function use its default
+        format = None
 
+    # Save to a file and display in the default viewer.
+    # We must close the file before viewing it.
+    graph.draw(path, format, prog, args)
+    path.close()
+    nx.utils.default_opener(filename)
+
+# fixture for nose tests
+def setup_module(module):
+    from nose import SkipTest
+    try:
+        import pygraphviz
+    except:
+        raise SkipTest("pygraphviz not available")
