@@ -7,19 +7,19 @@ Do not add this file to the repository.
 
 import datetime
 
-version = '1.5'
-date = 'Sat Jun  4 09:49:07 2011'
+version = '1.6'
+date = 'Tue Nov 22 07:15:35 2011'
 
 # Was NetworkX built from a development version? If so, remember that the major
 # and minor versions reference the "target" (rather than "current") release.
 dev = False
 
 # Format: (name, major, min, revision)
-version_info = ('networkx', 1, 5, '7534d8095e2c')
+version_info = ('networkx', 1, 6, '2049c85eac50')
 
 # Format: a 'datetime.datetime' instance
-date_info = datetime.datetime(2011, 6, 4, 9, 49, 7, 997782)
+date_info = datetime.datetime(2011, 11, 22, 7, 15, 35, 408614)
 
 # Format: (vcs, vcs_tuple)
-vcs_info = ('mercurial', ('7534d8095e2c', 'networkx-1.5'))
+vcs_info = ('mercurial', ('2049c85eac50', 'tip'))
 
